/* Includes ------------------------------------------------------------------*/
#include "DEV_Config.h"
#include "EPD.h"
#include "GUI_Paint.h"
#include "imagedata.h"
#include <stdlib.h>

/* Entry point ----------------------------------------------------------------*/
void setup()
{
  printf("EPD_2IN7_test Demo\r\n");
  DEV_Module_Init();

  printf("e-Paper Init and Clear...\r\n");
  EPD_2IN7_Init();
  EPD_2IN7_Clear();
  DEV_Delay_ms(500);

  //Create a new image cache
  UBYTE *BlackImage;
  /* you have to edit the startup_stm32fxxx.s file and set a big enough heap size */
  UWORD Imagesize = ((EPD_2IN7_WIDTH % 8 == 0) ? (EPD_2IN7_WIDTH / 8 ) : (EPD_2IN7_WIDTH / 8 + 1)) * EPD_2IN7_HEIGHT;
  if ((BlackImage = (UBYTE *)malloc(Imagesize)) == NULL) {
    printf("Failed to apply for black memory...\r\n");
    while (1);
  }
  printf("Paint_NewImage\r\n");
  Paint_NewImage(BlackImage, EPD_2IN7_WIDTH, EPD_2IN7_HEIGHT, 270, WHITE);


#if 1   // show image for array   
  printf("show image for array\r\n");
  Paint_SelectImage(BlackImage);
  Paint_Clear(WHITE);
  Paint_DrawBitMap(gImage_2in7);
  EPD_2IN7_Display(BlackImage);
  DEV_Delay_ms(500);
#endif

#if 1  // Drawing on the image
  //1.Select Image
  printf("SelectImage:BlackImage\r\n");
  Paint_SelectImage(BlackImage);
  Paint_Clear(WHITE);

  // 2.Drawing on the image
  printf("Drawing:BlackImage\r\n");
  Paint_DrawPoint(10, 80, BLACK, DOT_PIXEL_1X1, DOT_STYLE_DFT);
  Paint_DrawPoint(10, 90, BLACK, DOT_PIXEL_2X2, DOT_STYLE_DFT);
  Paint_DrawPoint(10, 100, BLACK, DOT_PIXEL_3X3, DOT_STYLE_DFT);
  Paint_DrawLine(20, 70, 70, 120, BLACK, DOT_PIXEL_1X1, LINE_STYLE_SOLID);
  Paint_DrawLine(70, 70, 20, 120, BLACK, DOT_PIXEL_1X1, LINE_STYLE_SOLID);
  Paint_DrawRectangle(20, 70, 70, 120, BLACK, DOT_PIXEL_1X1, DRAW_FILL_EMPTY);
  Paint_DrawRectangle(80, 70, 130, 120, BLACK, DOT_PIXEL_1X1, DRAW_FILL_FULL);
  Paint_DrawCircle(45, 95, 20, BLACK, DOT_PIXEL_1X1, DRAW_FILL_EMPTY);
  Paint_DrawCircle(105, 95, 20, WHITE, DOT_PIXEL_1X1, DRAW_FILL_FULL);
  Paint_DrawLine(85, 95, 125, 95, BLACK, DOT_PIXEL_1X1, LINE_STYLE_DOTTED);
  Paint_DrawLine(105, 75, 105, 115, BLACK, DOT_PIXEL_1X1, LINE_STYLE_DOTTED);
  Paint_DrawString_EN(10, 0, "waveshare", &Font16, BLACK, WHITE);
  Paint_DrawString_EN(10, 20, "hello world", &Font12, WHITE, BLACK);
  Paint_DrawNum(10, 33, 123456789, &Font12, BLACK, WHITE);
  Paint_DrawNum(10, 50, 987654321, &Font16, WHITE, BLACK);
  Paint_DrawString_CN(130, 0, "你好abc", &Font12CN, BLACK, WHITE);
  Paint_DrawString_CN(130, 20, "微雪电子", &Font24CN, WHITE, BLACK);

  printf("EPD_Display\r\n");
  EPD_2IN7_Display(BlackImage);
  DEV_Delay_ms(2000);
#endif
  printf("Clear...\r\n");
  // EPD_2IN7_Clear();
  free(BlackImage);

#if 1 // show image for array
  printf("show Gray------------------------\r\n");
  Imagesize = ((EPD_2IN7_WIDTH % 4 == 0) ? (EPD_2IN7_WIDTH / 4 ) : (EPD_2IN7_WIDTH / 4 + 1)) * EPD_2IN7_HEIGHT;
  if ((BlackImage = (UBYTE *)malloc(Imagesize)) == NULL) {
    printf("Failed to apply for black memory...\r\n");
    while (1);
  }
  printf("4 grayscale display\r\n");
  EPD_2IN7_Init_4Gray();
  Paint_NewImage(BlackImage, EPD_2IN7_WIDTH, EPD_2IN7_HEIGHT, 270, WHITE);
  Paint_SetScale(4);
  Paint_Clear(WHITE);

  Paint_DrawPoint(10, 80, GRAY4, DOT_PIXEL_1X1, DOT_STYLE_DFT);
  Paint_DrawPoint(10, 90, GRAY4, DOT_PIXEL_2X2, DOT_STYLE_DFT);
  Paint_DrawPoint(10, 100, GRAY4, DOT_PIXEL_3X3, DOT_STYLE_DFT);
  Paint_DrawLine(20, 70, 70, 120, GRAY4, DOT_PIXEL_1X1, LINE_STYLE_SOLID);
  Paint_DrawLine(70, 70, 20, 120, GRAY4, DOT_PIXEL_1X1, LINE_STYLE_SOLID);
  Paint_DrawRectangle(20, 70, 70, 120, GRAY4, DOT_PIXEL_1X1, DRAW_FILL_EMPTY);
  Paint_DrawRectangle(80, 70, 130, 120, GRAY4, DOT_PIXEL_1X1, DRAW_FILL_FULL);
  Paint_DrawCircle(45, 95, 20, GRAY4, DOT_PIXEL_1X1, DRAW_FILL_EMPTY);
  Paint_DrawCircle(105, 95, 20, GRAY2, DOT_PIXEL_1X1, DRAW_FILL_FULL);
  Paint_DrawLine(85, 95, 125, 95, GRAY4, DOT_PIXEL_1X1, LINE_STYLE_DOTTED);
  Paint_DrawLine(105, 75, 105, 115, GRAY4, DOT_PIXEL_1X1, LINE_STYLE_DOTTED);
  Paint_DrawString_EN(10, 0, "waveshare", &Font16, GRAY4, GRAY1);
  Paint_DrawString_EN(10, 20, "hello world", &Font12, GRAY3, GRAY1);
  Paint_DrawNum(10, 33, 123456789, &Font12, GRAY4, GRAY2);
  Paint_DrawNum(10, 50, 987654321, &Font16, GRAY1, GRAY4);
  Paint_DrawString_CN(150, 0, "你好abc", &Font12CN, GRAY4, GRAY1);
  Paint_DrawString_CN(150, 20, "你好abc", &Font12CN, GRAY3, GRAY2);
  Paint_DrawString_CN(150, 40, "你好abc", &Font12CN, GRAY2, GRAY3);
  Paint_DrawString_CN(150, 60, "你好abc", &Font12CN, GRAY1, GRAY4);
  Paint_DrawString_CN(10, 130, "微雪电子", &Font24CN, GRAY1, GRAY4);
  EPD_2IN7_4GrayDisplay(BlackImage);
  DEV_Delay_ms(3000);

  EPD_2IN7_4GrayDisplay(gImage_2in7_4Gray1);
  DEV_Delay_ms(3000);

#endif
  // close 5V
  printf("Goto Sleep...\r\n");
  EPD_2IN7_Init();
  EPD_2IN7_Clear();
  EPD_2IN7_Sleep();
  free(BlackImage);
  BlackImage = NULL;

}

/* The main loop -------------------------------------------------------------*/
void loop()
{
  //
}
